package vin.vio.collaboraonline.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.io.UrlResource;
import org.springframework.stereotype.Service;
import vin.vio.collaboraonline.model.WopiFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.nio.file.*;

/**
 * @Classname FileService
 * @Description TODO
 * @Date 2025/2/19 16:54
 * @Created by violet
 */

@Service
public class FileService {


    private static final String BASE_DIR = "E:\\Code\\CollaboraOnline\\files";


    public WopiFile getFileInfo(String fileId) {
        Path filePath = Paths.get(BASE_DIR, fileId);

        File targetFile = filePath.toFile();

        if (!targetFile.exists() || !targetFile.canRead() || !targetFile.canWrite()) {
            throw new RuntimeException("File not accessible: " + filePath);
        }

        WopiFile file = new WopiFile();
        try {
            file.setFileId(fileId);
            file.setFileName(targetFile.getName());
            file.setSize(targetFile.length());
            file.setFilePath(filePath);

            file.setOwnerId("admin");
            file.setUserCanWrite(true);
            file.setSupportsLocks(true);

            String contentType = Files.probeContentType(filePath);
            file.setContentType(contentType != null ? contentType : "application/octet-stream");

        } catch (IOException e) {
            throw new RuntimeException("Error reading file metadata: " + filePath, e);
        }

        return file;
    }

    public Resource loadFileAsResource(String fileId) {
        try {
            Path resolvedPath = validateFilePath(fileId);

            FileSystemResource resource = new FileSystemResource(resolvedPath);

            if (resource.exists() && resource.isReadable() && Files.isRegularFile(resolvedPath)) {
                return resource;
            }
            return null;
        } catch (InvalidPathException | SecurityException e) {
            throw new RuntimeException("非法文件访问: " + fileId, e);
        }
    }

    public boolean saveFile(String fileId, InputStream content) {
        Path targetPath = validateFilePath(fileId);

        try {
            Path tempFile = Files.createTempFile(targetPath.getParent(), "wopi_", ".tmp");

            try (OutputStream os = Files.newOutputStream(tempFile, StandardOpenOption.WRITE)) {
                byte[] buffer = new byte[8192];
                int bytesRead;
                while ((bytesRead = content.read(buffer)) != -1) {
                    os.write(buffer, 0, bytesRead);
                }
            }

            Files.move(tempFile, targetPath, StandardCopyOption.ATOMIC_MOVE, StandardCopyOption.REPLACE_EXISTING);

            setFilePermissions(targetPath);

            return true;
        } catch (IOException e) {
            throw new RuntimeException("文件保存失败: " + targetPath, e);
        }
    }

    private Path validateFilePath(String fileId) {
        if (fileId.contains("..") || fileId.contains(":") || fileId.contains("\\\\")) {
            throw new SecurityException("非法文件名: " + fileId);
        }

        Path basePath = Paths.get(BASE_DIR).normalize().toAbsolutePath();
        Path resolvedPath = basePath.resolve(fileId).normalize();

        if (!resolvedPath.startsWith(basePath)) {
            throw new SecurityException("路径越界访问: " + fileId);
        }

        return resolvedPath;
    }

    private void setFilePermissions(Path path) throws IOException {
        if (!System.getProperty("os.name").toLowerCase().contains("win")) return;

        String aclCommand = String.format(
                "icacls \"%s\" /grant:r \"%s\":(F) /inheritance:r",
                path.toString(),
                System.getProperty("user.name")
        );

        try {
            Runtime.getRuntime().exec(aclCommand).waitFor();
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }
}
